Object.defineProperty(exports, "__esModule", { value: true });
exports.HTTPTransport = void 0;
var tslib_1 = require("tslib");
var types_1 = require("@amplitude/types");
var http = require("http");
var https = require("https");
// Automatially cancel requests that have been waiting for 10+ s
var REQUEST_CANCEL_TIMEOUT = 10 * 1000;
/** Base Transport class implementation */
var HTTPTransport = /** @class */ (function () {
    /** Create instance and set this.dsn */
    function HTTPTransport(options) {
        this.options = options;
        this._uploadInProgress = false;
        this._requestQueue = [];
        if (options.serverUrl.startsWith('http://')) {
            this.module = http;
        }
        else if (options.serverUrl.startsWith('https://')) {
            this.module = https;
        }
        else {
            throw new Error('Invalid server url');
        }
    }
    /**
     * @inheritDoc
     */
    HTTPTransport.prototype.sendPayload = function (payload) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var call;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                call = function () { return _this._sendWithModule(payload); };
                // Queue up the call to send the payload.
                // Wait 10 seconds for each request in queue before removing it
                return [2 /*return*/, this._awaitUploadFinish(call, REQUEST_CANCEL_TIMEOUT)];
            });
        });
    };
    /** Returns a build request option object used by request */
    HTTPTransport.prototype._getRequestOptions = function () {
        var headers = tslib_1.__assign({}, this.options.headers);
        var url = new URL(this.options.serverUrl);
        var options = {
            headers: headers,
            method: 'POST',
            hostname: url.hostname,
            path: url.pathname,
        };
        return options;
    };
    // Awaits the finish of all requests that have been queued up before it
    // And will expire itself (reject the promise) after waiting limit ms
    // or never expire, if limit is not set
    HTTPTransport.prototype._awaitUploadFinish = function (callback, limitInMs) {
        var _this = this;
        if (limitInMs === void 0) { limitInMs = 0; }
        return new Promise(function (resolve, reject) {
            var queueCallback = function () {
                _this._uploadInProgress = true;
                try {
                    resolve(callback());
                }
                catch (e) {
                    reject(e);
                }
                finally {
                    _this._notifyUploadFinish();
                }
            };
            // If there is no upload in progress
            // Return immediately
            if (!_this._uploadInProgress) {
                return queueCallback();
            }
            var requestObject = {
                callback: queueCallback,
                cancellingTimeout: null,
            };
            _this._requestQueue.push(requestObject);
            // If the limit exists, set a timeout to remove the callback and reject the promise
            if (limitInMs > 0) {
                requestObject.cancellingTimeout = setTimeout(function () {
                    var callBackIndex = _this._requestQueue.findIndex(function (requestObj) {
                        return requestObj.callback === queueCallback;
                    });
                    if (callBackIndex > -1) {
                        _this._requestQueue.splice(callBackIndex, 1);
                    }
                    reject();
                }, limitInMs);
            }
        });
    };
    // Notify the oldest awaiting send that the transport is ready for another request
    HTTPTransport.prototype._notifyUploadFinish = function () {
        this._uploadInProgress = false;
        var oldestRequest = this._requestQueue.shift();
        if (oldestRequest) {
            if (oldestRequest.cancellingTimeout !== null) {
                // Clear the timeout where we try to remove the callback and reject the promise.
                clearTimeout(oldestRequest.cancellingTimeout);
            }
            oldestRequest.callback();
        }
    };
    /** JSDoc */
    HTTPTransport.prototype._sendWithModule = function (payload) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        var req = _this.module.request(_this._getRequestOptions(), function (res) {
                            res.setEncoding('utf8');
                            var rawData = '';
                            // Collect the body data from the response
                            res.on('data', function (chunk) {
                                rawData += chunk;
                            });
                            // On completion, parse the data and resolve.
                            res.on('end', function () {
                                if (res.complete && rawData) {
                                    try {
                                        var responseWithBody = types_1.mapJSONToResponse(JSON.parse(rawData));
                                        if (responseWithBody !== null) {
                                            return resolve(responseWithBody);
                                        }
                                    }
                                    catch (_a) { }
                                }
                                // Fallback: get the response object directly from the incoming message
                                resolve(types_1.mapHttpMessageToResponse(res));
                            });
                        });
                        req.on('error', reject);
                        req.end(JSON.stringify(payload));
                    })];
            });
        });
    };
    return HTTPTransport;
}());
exports.HTTPTransport = HTTPTransport;
//# sourceMappingURL=http.js.map